# CREATE INDEX index_name ON table.name (name)
# [{'CreateIndex': {'name': [{'value': 'index', 'quote_style': None}], 'table_name': [{'value': 'table', 'quote_style': None}, {'value': 'name', 'quote_style': None}], 'columns': [{'expr': {'Identifier': {'value': 'name', 'quote_style': None}}, 'asc': None, 'nulls_first': None}], 'unique': False, 'if_not_exists': False}}]
